@@ -6,7 +6,7 @@ fn basic_cases() {
     let cnt = 200;
 
     let hdr = {
-        let mut hdr_i = MapxRaw::new();
+        let hdr_i = MapxRaw::new();
 
         assert_eq!(0, hdr_i.len());
         (0..cnt).for_each(|i: usize| {
@@ -29,7 +29,7 @@ fn basic_cases() {
         <MapxRaw as ValueEnDe>::encode(&hdr_i)
     };
 
-    let mut reloaded = pnk!(<MapxRaw as ValueEnDe>::decode(&hdr));
+    let reloaded = pnk!(<MapxRaw as ValueEnDe>::decode(&hdr));
 
     assert_eq!(cnt, reloaded.len());
 
