@@ -1,8 +1,11 @@
 //!
-//! This is a simple merkle-tree implementation ported from solana project.
+//! A simple 'Merkle-Tree' ported from solana project.
 //!
 
-use crate::{basic::vecx::Vecx, common::RawBytes};
+use crate::{
+    basic::{mapx_ord_rawkey::MapxOrdRawKey, vecx_raw::VecxRaw},
+    common::RawBytes,
+};
 use serde::{Deserialize, Serialize};
 use sha3::{Digest, Sha3_256};
 
@@ -30,7 +33,7 @@ macro_rules! hash_leaf {
 
 macro_rules! hash_intermediate {
     {$l:ident, $r:ident} => {
-        hashv(&[INTERMEDIATE_PREFIX, $l.as_ref(), $r.as_ref()])
+        hashv(&[INTERMEDIATE_PREFIX, $l, $r])
     }
 }
 
@@ -38,6 +41,7 @@ macro_rules! hash_intermediate {
 pub struct MerkleTree {
     leaf_count: usize,
     nodes: Vec<Hash>,
+    hash_to_idx: MapxOrdRawKey<u64>,
 }
 
 #[derive(Debug, PartialEq)]
@@ -75,7 +79,7 @@ impl<'a> Proof<'a> {
 }
 
 impl MerkleTree {
-    #[inline]
+    #[inline(always)]
     fn next_level_len(level_len: usize) -> usize {
         if level_len == 1 {
             0
@@ -84,6 +88,7 @@ impl MerkleTree {
         }
     }
 
+    #[inline(always)]
     fn calculate_vec_capacity(leaf_count: usize) -> usize {
         // the most nodes consuming case is when n-1 is full balanced binary tree
         // then n will cause the previous tree add a left only path to the root
@@ -107,16 +112,17 @@ impl MerkleTree {
         }
     }
 
-    pub fn new<T: AsRef<[u8]>>(items: &[T]) -> Self {
+    pub fn new(items: &[&[u8]]) -> Self {
         let cap = MerkleTree::calculate_vec_capacity(items.len());
         let mut mt = MerkleTree {
             leaf_count: items.len(),
             nodes: Vec::with_capacity(cap),
+            hash_to_idx: MapxOrdRawKey::new(),
         };
 
-        for item in items {
-            let item = item.as_ref();
+        for (idx, item) in items.iter().enumerate() {
             let hash = hash_leaf!(item);
+            mt.hash_to_idx.insert_ref(&hash, &(idx as u64));
             mt.nodes.push(hash);
         }
 
@@ -147,11 +153,24 @@ impl MerkleTree {
         mt
     }
 
+    #[inline(always)]
     pub fn get_root(&self) -> Option<&Hash> {
         self.nodes.iter().last()
     }
 
-    pub fn find_path(&self, index: usize) -> Option<Proof> {
+    #[inline(always)]
+    pub fn get_proof_path(&self, target: &[u8]) -> Option<Proof> {
+        let hash = hashv(&[target]);
+        self.get_proof_path_by_hash(&hash)
+    }
+
+    #[inline(always)]
+    pub fn get_proof_path_by_hash(&self, target_hash: &[u8]) -> Option<Proof> {
+        let idx = self.hash_to_idx.get(target_hash)? as usize;
+        self.get_proof_path_by_idx(idx)
+    }
+
+    pub fn get_proof_path_by_idx(&self, index: usize) -> Option<Proof> {
         if index >= self.leaf_count {
             return None;
         }
@@ -192,27 +211,32 @@ impl MerkleTree {
 #[derive(Clone, Debug, Deserialize, Serialize)]
 pub struct MerkleTreeStore {
     leaf_count: usize,
-    nodes: Vecx<Hash>,
+    nodes: VecxRaw,
+    hash_to_idx: MapxOrdRawKey<u64>,
 }
 
 impl From<&MerkleTree> for MerkleTreeStore {
+    #[inline(always)]
     fn from(mt: &MerkleTree) -> Self {
-        let nodes = Vecx::new();
+        let nodes = VecxRaw::new();
         mt.nodes.iter().for_each(|h| {
             nodes.push_ref(h);
         });
         Self {
             leaf_count: mt.leaf_count,
             nodes,
+            hash_to_idx: mt.hash_to_idx,
         }
     }
 }
 
 impl From<&MerkleTreeStore> for MerkleTree {
+    #[inline(always)]
     fn from(mts: &MerkleTreeStore) -> Self {
         Self {
             leaf_count: mts.leaf_count,
             nodes: mts.nodes.iter().collect(),
+            hash_to_idx: mts.hash_to_idx,
         }
     }
 }
@@ -229,7 +253,7 @@ mod tests {
 
     #[test]
     fn test_tree_from_empty() {
-        let mt = MerkleTree::new::<[u8; 0]>(&[]);
+        let mt = MerkleTree::new(&[]);
         assert_eq!(mt.get_root(), None);
     }
 
@@ -245,14 +269,14 @@ mod tests {
     fn test_path_creation() {
         let mt = MerkleTree::new(TEST);
         for (i, _s) in TEST.iter().enumerate() {
-            let _path = mt.find_path(i).unwrap();
+            let _path = mt.get_proof_path_by_idx(i).unwrap();
         }
     }
 
     #[test]
     fn test_path_creation_bad_index() {
         let mt = MerkleTree::new(TEST);
-        assert_eq!(mt.find_path(TEST.len()), None);
+        assert_eq!(mt.get_proof_path_by_idx(TEST.len()), None);
     }
 
     #[test]
@@ -260,7 +284,7 @@ mod tests {
         let mt = MerkleTree::new(TEST);
         for (i, s) in TEST.iter().enumerate() {
             let hash = hash_leaf!(s);
-            let path = mt.find_path(i).unwrap();
+            let path = mt.get_proof_path_by_idx(i).unwrap();
             assert!(path.verify(hash));
         }
     }
@@ -270,7 +294,7 @@ mod tests {
         let mt = MerkleTree::new(TEST);
         for (i, s) in BAD.iter().enumerate() {
             let hash = hash_leaf!(s);
-            let path = mt.find_path(i).unwrap();
+            let path = mt.get_proof_path_by_idx(i).unwrap();
             assert!(!path.verify(hash));
         }
     }
